function [Segmented, Clusters]=DSNew(A,Method,Flag, IntensityCorrection, NoIterativeProcess)
try
tic;

% This function applies the Delaunay-based image segmentation, which is a fully automated process that does not require initial estimate of number of clusters.
% The core idea is to apply Delaunay triangulation to the image histogram instead of the image itself. That reduces the sites required to construct the diagram to 
% merely 255 at most (uint8) resulting in a fast image segmentation.
% I don't claim it is the optimal way to segment an image, which is why I will be more than happy receiving constructive comments or reporting any bug in the program for further improvement.
%
% For a detailed description of the theoretical foundation of the algorithm and for citation purposes please refer to the journal paper -a pre-print copy is included here but it is better to have it in PR layout-:
%
% - A. Cheddad, D. Mohamad and A. Abd Manaf, "Exploiting Voronoi diagram properties in face segmentation and features extraction," Pattern Recognition, 41 (12)(2008)3842-3859, Elsevier Science.
% - A. Cheddad, J. Condell, K. Curran and P. Mc Kevitt. On Points Geometry for Fast Digital Image Segmentation. The 8th International Conference on Information Technology and Telecommunication IT&T 2008, 
%   Ireland 23  24 October 2008, pp: 54-61.
 
% Usage
% [Segmented, Clusters]=DSNew(A,1,0,1,1);

% Where A is your grayscale/RGB image, its intensity is expected to range from 0 to 255
% 8-bit.

% Method: This is mainly to pre-process color images "1" for rgb2gray and
% "0" for extracting and using the luminance from the HSV colorspace

% Flag: To work on the imcomplement of the image. "1" Do, "0" Don't.

% IntensityCorrection: Replace a clustered region value with the mean intensity of
% the corresponding region in the original image.

% NoIterativeProcess: This is to be set to "1" unless you invoke the
% iterative version. Its syntax is as follows:
% [Segmented,Clusters]=DSIterate(A,Method,Flag,IntensityCorrection);
% [Segmented,Clusters]=DSIterate(A,1,0,1);
% NoIterativeProcess is set to "0" from within this function. This function
% keeps segmenting the output until it converges and that happens when the
% clusters can no longer be further decomposed.

%Please keep in mind that this function is meant to segment a given image
%without having access to priori information on the number of clusters. It approximates a
%good segmentation but I don't claim it is the optimum. One, however, could use the number of clusters that it arrives
%at as an input to a more sphisticated (time consuming) segmentation algorithms like the K-means for example.


Color=[];


if length(size(A))==3
    Color=A;
if Method==1
    A=rgb2gray(A);
    
else
    
    A=rgb2hsv(A);
    A=im2uint8(A(:,:,3));
    
   
    %A=histeq(A);
end
end
%else
    

%%%%%%%%%Preprocessing
Store=A;

if Flag==1
    A=imcomplement(A);
end

F(:,:,1)=im2double(medfilt2(A));F(:,:,2)=im2double((histeq(medfilt2(imadjust(A)))));F(:,:,3)=zeros (size(im2double((A))));
FF=rgb2hsv(F);
A=(im2uint8(FF(:,:,3)));

%end
%%%%%%%%%%%%%%%%%%%%%%




% if resize==1
%     A=imresize(A,0.25);
% end

[x xx]=imhist(im2uint8(A));
C=[x,xx];

L=x;
[ind1 ind]=max(x);
for j=1:256
[ind2 ~]=find(x==x(j));
ind2=ind2(1);
dis=sqrt(ind.^2+ind2.^2);
x(j)=x(j)*dis;
end

k=convhull(C(:,1),C(:,2));
X_conv=C(k,1);
Y_conv=C(k,2);
C2=[x,xx];
k2=convhull(C2(:,1),C2(:,2));
X_conv2=C2(k2,1);
Y_conv2=C2(k2,2);
K=[k;k2];
K=unique(K);
K5=[0;K]; 
P=[K,diff(K5)];
[y yy]=find(P(:,2)==1);

%P(y(2:end),1)=P(y(2:end)-1,1)
for j=2:length(y)
P(y(j),1)=P(y(j)-1,1);
end
P=unique(sort(P));
[m mm]=bwlabel(diff(P)==1);
P=check(P,mm,m);
while mm~=0
   
    for t=1:mm
       [x xx]=find(m==t);
       P(x)=round(mean(P(m==t,1)));
             
    end
P=unique(sort(P));
P=check(P,mm,m);
[m mm]=bwlabel(diff(P)==1);
end

P=sort(unique(P));

% 

[v vv]=max(x);
L(1:min(ind,vv),1)=ind1;
L(max(ind,vv):256,1)=ind1;
L=(abs(L-max(L)));
C(:,1)=L;
f=convhull(C(:,1),C(:,2));
f=(diff(f));
f(f<=1)=[];
PP=unique([P;f]);


K5=[0;PP]; 
P3=[PP,diff(K5)];
[y yy]=find(P3(:,2)==1);

%P(y(2:end),1)=P(y(2:end)-1,1)
for j=2:length(y)
P3(y(j),1)=P3(y(j)-1,1);
end
P3=unique(P3);



[m mm]=bwlabel(diff(P3)==1);
P3=check(P3,mm,m);
while mm~=0
   
    for t=1:mm
       [x xx]=find(m==t);
       P3(x)=round(mean(P3(m==t,1)));
             
    end
P3=unique(sort(P3));
P3=check(P3,mm,m);
[m mm]=bwlabel(diff(P3)==1);
end

P3=sort(unique(P3));

catch err 

P3=P;
sprintf('P selected')
end

%P  PP P3
Array=P3;
Orig=im2uint8(A);
Orig(Orig<Array(1,1))=0;
Orig(Orig>Array(length(Array(:)),1))=max(max(A));
%d=[];
Im=Orig;
%%%%%%%%%%%%%%%%%%%%Segmentation of the image%%%%%%%%%%%%%%%
for i=1:length(Array(:))-1
   
    if i==1
    Orig(Orig>=Array(i) & Orig<=Array(i+1))=Array(i+1);
    %d=[d;Array(i+1)];
       else
    Orig(Orig>Array(i) & Orig<=Array(i+1))=Array(i+1);
    %d=[d;Array(i+1)];
    end
    
  
end

%d=sort(d);
Orig_Color=[];

if length(size(Im))==3
Im(:,:,1)=im2double(Orig);
Orig_Color=ntsc2rgb(Im);
[x xx]=imhist(im2uint8(Im(:,:,1)));
Array=xx(x>0);

else
    
[x xx]=imhist(im2uint8(Orig(:,:,1)));% Corrected!

Array=xx(x>0);

end

Segmented=im2uint8(Orig);

Time=toc;

% Calculating additional info, i.e., comparision of the number of 
% grayvalues before and after the segmentation


[x ~]=imhist(Store);
Count=x(x>0);

[n ~]=imhist(Segmented);
L=find(n>0);
Clusters=L-1;

if IntensityCorrection==1
SegmentedCopy=Segmented;  
    for Ind=1:(length(Clusters)-1)
        Val= ((Segmented>=Clusters(Ind) & Segmented<Clusters(Ind+1)).*im2double(Store));
        Val=mean(Val(Val>0));
        SegmentedCopy(Segmented>=Clusters(Ind) & Segmented<Clusters(Ind+1))=im2uint8(Val);
        
    end 
Segmented=SegmentedCopy;
end

[n ~]=imhist(Segmented);
L=find(n>0);
Clusters=L-1;

    
if NoIterativeProcess==1
    
    figure, imshow(Segmented), title(sprintf('Delaunay based unsupervised segmentation, excusion time: %s Sec',num2str(Time))), %colormap(hot)
if  ~isempty(Color)
Color=rgb2hsv(Color);
Color(:,:,3)=im2double(Segmented);
figure, imshow((hsv2rgb(Color))),title(sprintf('Color Delaunay based unsupervised segmentation , excusion time: %s Sec',num2str(Time)));
imwrite((hsv2rgb(Color)),'Segmented_Color.tif')
end
msgbox(sprintf('The number of gray values in the original was: %d which is reduced to: %d with the segmentation process.',length(Count),length(Array)),'Title','modal');
end







